#!/usr/bin/python3
#
# "Static" commands
#

import miniirc, os, sys, lurklite.tempcmds as tempcmds, time

commands = {}

# Register commands
def register_command(*cmds, with_bot=False, requires_admin=False):
    def n(func):
        if requires_admin:
            def wrap_cmd(bot, irc, hostmask, is_admin, args):
                if not is_admin:
                    irc.msg(args[0], 'Permission denied!')
                elif with_bot:
                    return func(bot, irc, hostmask, is_admin, args)
                else:
                    return func(irc, hostmask, is_admin, args)
            wrap_cmd._lurklite_self = True
        else:
            wrap_cmd = func
            if with_bot:
                wrap_cmd._lurklite_self = True

        for cmd in cmds:
            commands[cmd.lower()] = wrap_cmd

    return n

# Load custom commands
def load_cmd_file(file, *, recursive=True):
    if recursive and os.path.isdir(file):
        for f in os.listdir(file):
            if f.endswith('.py'):
                load_cmd_file(os.path.join(file, f), recursive=False)
        return

    # Hacks to add a global variable
    try:
        with open(file, 'r') as f:
            custom_cmds = f.read()
    except:
        print('WARNING: Failed to read a custom commands file:', repr(file))
        return

    module = type(tempcmds)('custom_cmds')
    module.__file__ = file
    module.register_command = register_command
    exec(custom_cmds, module.__dict__)

# A simple version command
@register_command('version')
def _cmd_version(irc, hostmask, is_admin, args):
    irc.msg(args[0], miniirc.version)

# Reboot
@register_command('reboot', requires_admin=True)
def _cmd_reboot(irc, hostmask, is_admin, args):
    irc.notice(args[0], '\x037\x1dRebooting...\x1d')
    print(is_admin, 'ordered me to reboot.')
    time.sleep(0.3)
    argv = (sys.executable, '-m', 'lurklite', *sys.argv[1:])

    if os.name == 'posix':
        # On POSIX systems, just use execvp().
        os.execvp(sys.executable, argv)
    else:
        # Otherwise spawn a child process because Windows has a horribly
        # broken os.execvp implementation.
        import subprocess
        subprocess.Popen(argv)
        os._exit(0)

# Shutdown
@register_command('die', 'shutdown', requires_admin=True)
def _cmd_die(irc, hostmask, is_admin, args):
    irc.notice(args[0], '\x037\x1dGoodbye, cruel world!\x1d')
    print(is_admin, 'ordered me to die.')
    time.sleep(0.3)
    os._exit(0)

# Privs
@register_command('privs', 'privileges')
def _cmd_privs(irc, hostmask, is_admin, args):
    if is_admin:
        irc.msg(args[0], f'{hostmask[0]}: You are an admin: `{is_admin}`.')
    else:
        irc.msg(args[0], f'{hostmask[0]}: You are not an admin!')

# Get a tempcmd name
def _get_tempcmd_name(bot, cmd):
    prefix = bot.cmd_db.prefix
    if cmd.startswith(prefix):
        r_cmd = repr(cmd)
        cmd   = cmd[len(prefix):]
    else:
        r_cmd = repr(prefix + cmd)

    return cmd, r_cmd

# Add and remove "tempcmds"
@register_command('tempcmd', 'tempcmds', with_bot=True, requires_admin=True)
def _cmd_tempcmd(bot, irc, hostmask, is_admin, args):
    """
    Creates a "tempcmd".
    Usage: tempcmd del <command>
    """

    tempcmd_db = bot.cmd_db
    assert tempcmd_db

    # Handle the arguments
    cmd_type = None
    params = args[-1].split(' ', 2)

    if len(params) > 1 and params[0] == 'add':
        cmd_type = False

        if len(params) == 3:
            params = [params[1]] + params[2].split(' ', 1)
        else:
            del params[0]

    if len(params) == 3:
        if tempcmds.command_type_exists(params[1]):
            cmd, cmd_type, code = params
        else:
            cmd, code = params[0], ' '.join(params[1:])
    elif len(params) == 2:
        cmd, code = params
    else:
        return irc.msg(args[0], hostmask[0] + ': Invalid syntax!')

    log = bot._prefs.get(irc, {}).get('tempcmd_log')

    # Get tempcmd info
    if cmd_type is None and cmd == 'info':
        cmd, r_cmd = _get_tempcmd_name(bot, code)

        if cmd not in tempcmd_db:
            return irc.msg(args[0], hostmask[0] + ': The command '
                + r_cmd + ' does not exist or is not a tempcmd!')

        data = tempcmd_db.get(cmd, allowed_aliases=0)

        return irc.msg(args[0], f'{hostmask[0]}: The command {r_cmd} is a '
            f'{data.type!r} tempcmd.\nCode: `{data.code}`')

    # Delete tempcmds
    if cmd_type is None and cmd in ('del', 'delete', 'remove'):
        cmd, r_cmd = _get_tempcmd_name(bot, code)

        if cmd not in tempcmd_db:
            return irc.msg(args[0], hostmask[0] + ': The command '
                + r_cmd + ' does not exist or is not a tempcmd!')

        del tempcmd_db[cmd]
        if log:
            irc.msg(log, f'User {is_admin!r} deleted temporary command '
                f'{r_cmd}.')
        irc.msg(args[0], hostmask[0] + ': Command ' + r_cmd + ' deleted.')
        return

    # Make sure the command does not start with the prefix
    cmd, r_cmd = _get_tempcmd_name(bot, cmd)

    # Make sure the command is not a non-tempcmd
    if cmd.lower() in commands:
        return irc.msg(args[0], hostmask[0] + ': The command ' + r_cmd +
            ' already exists as a normal command!')

    # Add the command
    verb = 'updated' if cmd in tempcmd_db else 'created'
    c = {'code': code}
    if cmd_type:
        c['type'] = cmd_type
    tempcmd_db[cmd] = c

    # Get the type
    if not cmd_type:
        cmd_type = tempcmd_db.get(cmd, allowed_aliases=0).type

    # Return the message
    if log:
        irc.msg(log, f'User {is_admin!r} {verb} temporary command {r_cmd} '
            f'(of type {cmd_type!r}): {code!r}')
    irc.msg(args[0], f'{hostmask[0]}: Command {r_cmd} (of type {cmd_type!r}) '
        f'{verb}.')



#!/usr/bin/python3
#
# lurklite: A "lightweight™" version of lurk.
#

import argparse, configparser, miniirc, sys
import lurklite.core as core

# Process arguments
def main():
    parser = argparse.ArgumentParser(prog='lurklite')
    parser.add_argument('config_file',
        help='The config file to use with lurklite.')
    parser.add_argument('--verbose', '--debug', action='store_true',
        help='Enable verbose/debugging mode.')
    parser.add_argument('-v', '--version', action='version',
        version=miniirc.version)
    args = parser.parse_args()

    # Load the config file
    config = configparser.ConfigParser()
    config.read(args.config_file)

    # Create the bot
    try:
        bot = core.Bot(config, debug=args.verbose)
    except core.BotError as e:
        print(f'ERROR: {e}', file=sys.stderr)
        raise SystemExit(1)

    bot.wait_until_disconnected()

# Call main() if required.
if __name__ == '__main__':
    main()



#!/usr/bin/python3
#
# lurklite core
#

import concurrent.futures, os, miniirc, re, sys, time
import lurklite.tempcmds as tempcmds
static_cmds = None

# The version
miniirc.version = f'lurklite v0.4.25 (powered by {miniirc.version})'

# Throw errors
class BotError(Exception):
    pass

def err(msg, *args, **kwargs):
    if len(args) > 0 or len(kwargs) > 0:
        msg = msg.format(*args, **kwargs)
    raise BotError(msg)

# The bot class
class Bot:
    # Make sure config values exist
    def _conf_assert(self, section, *keys):
        for key in keys:
            req = None
            if type(key) == tuple:
                key, req = key

            if key not in self.config[section]:
                err('Required config value {} missing (in section {})!',
                    repr(key), repr(section))
            elif req:
                try:
                    req(self.config[section][key])
                except:
                    err('Config value {} (in section {}) contains an invalid '
                        '{}.', repr(key), repr(section), req.__name__)

    # Make sure booleans are valid
    def _conf_bool(self, section, key, default=None):
        try:
            return self.config[section].getboolean(key, default)
        except:
            err(f'Config value {key!r} (in section {section!r}) contains an '
                f'invalid boolean.')

    # Convert an ignores list into a RegEx
    def process_ignores(self, section):
        res = set()
        for victim in self.config[section].get('ignored', '').split(','):
            victim = victim.strip()
            if victim:
                res.add(re.escape(victim).lower().replace('\\*', '.*'))

        regex_ignore = self.config[section].get('regex_ignore', '').strip()
        if regex_ignore:
            res.add(regex_ignore)
        return re.compile('^(' + ')|('.join(res) + ')$', re.IGNORECASE)

    # Add extra items
    def _add_extras(self, section, c, irc):
        p = {}
        self._prefs[irc] = p

        # Add message handlers
        irc.Handler('PRIVMSG', colon=False)(self.handle_privmsg)
        if section != 'discord':
            irc.Handler('INVITE', colon=False)(self._handle_invite)

        # Process the ignores list
        if 'ignored' in c:
            p['ignored'] = self.process_ignores(section)

        # Process the admins list
        if 'admins' in c:
            p['admins'] = set()
            for admin in c['admins'].split(','):
                p['admins'].add(admin.lower().strip())

        # Add the tempcmds log channel
        if 'tempcmd_log' in c:
            p['tempcmd_log'] = c['tempcmd_log']

        if self._conf_bool(section, 'auto_accept_invites', False):
            p['auto_accept_invites'] = True

    # Handle PRIVMSGs
    def handle_privmsg(self, irc, hostmask, args):
        # Check for ignored users
        h = '{}!{}@{}'.format(*hostmask)
        _ignores = self._prefs[irc].get('ignored')
        if self.ignores.match(h) or (_ignores and _ignores.match(h)):
            return

        # Handle PMs correctly
        if args[0].lower() == irc.current_nick.lower():
            args[0] = hostmask[0]

        # [off] handling
        msg = args[-1]
        reply_prefix = ''
        if msg.startswith('[off]'):
            reply_prefix = '[off] '
            msg = msg[5:]

        # Relayed nick handling
        if msg.startswith('<'):
            n = msg.split(' ', 1)
            if n[0].endswith('>') and len(n) > 1 and msg[1].isalnum():
                _nick = n[0][1:-1]
                hostmask = (
                    _nick + '@' + hostmask[0],
                    hostmask[1],
                    hostmask[2] + '/relayed/' + _nick
                )
                msg = n[1]

        # Remove any leading/trailing spaces
        msg      = msg.strip(' \t\r\n')
        args[-1] = msg
        msg      = msg.lower()

        # Unprefixed commands here
        if not self.disable_yay and msg.startswith('yay'):
            irc.msg(args[0], reply_prefix + '\u200bYay!')
        elif not self.disable_ouch and msg.startswith('ouch'):
            irc.msg(args[0], reply_prefix + '\u200bOuch.')
        elif msg.startswith(irc.current_nick.lower() + '!'):
            irc.msg(args[0], reply_prefix + hostmask[0] + '!')
        else:
            # "Static" commands
            prefix = self.cmd_db.prefix
            if msg.startswith(prefix) and self.static_cmds:
                cmd = msg[len(prefix):].split(' ', 1)[0]

                if cmd in static_cmds.commands:
                    # Decide if the user is an admin
                    admins = self._prefs[irc].get('admins', ())
                    host = hostmask[2]

                    if type(irc).__name__ == 'Discord':
                        # Discord privileges are checked against both the user
                        # ID and username#discriminator.
                        if (host.startswith('discord/user/<') and
                                host[15:-1] in admins):
                            # Admin from user ID
                            is_admin = host[15:-1]
                        elif ('#' in hostmask[1] and
                                hostmask[1].lower() in admins):
                            # Admin from username#discriminator
                            is_admin = hostmask[1]
                        else:
                            # Not an admin
                            is_admin = False
                    else:
                        # IRC privileges are just checked against the hostname.
                        is_admin = host.lower() in admins and host

                    # Launch the command
                    args[-1] = args[-1][len(cmd) + len(prefix) + 1:]
                    func = static_cmds.commands[cmd]
                    if hasattr(func, '_lurklite_self'):
                        return func(self, irc, hostmask, is_admin, args)
                    else:
                        return func(irc, hostmask, is_admin, args)

            # Call the command handler
            self.cmd_db(irc, hostmask, args, reply_prefix=reply_prefix or None)

        # Update the Discord server count
        if 'next_update' in self._prefs[irc] and \
                time.time() > self._prefs[irc]['next_update']:
            c = irc.get_server_count()
            irc.quote('AWAY', ':{} guild{}. | {}help'.format(
                c, '' if c == 1 else 's', self.cmd_db.prefix
            ), tags = {'+discordapp.com/type': 'watching'})
            irc.debug('Updated Discord status text.')
            self._prefs[irc]['next_update'] = time.time() + 60

    # Accept invites from admins
    def _handle_invite(self, irc, hostmask, args):
        prefs = self._prefs[irc]
        if (prefs.get('auto_accept_invites') or
                hostmask[2].lower() in prefs.get('admins', ())):
            irc.send('JOIN', args[-1])

    # The init function
    def __init__(self, config, *, debug=False):
        self.config = config
        if 'core' not in config:
            err('Invalid or non-existent config file!')
        self._conf_assert('core', 'command_db', 'prefix')
        self.ignores = self.process_ignores('core')
        self._prefs = {}

        # Create the commands database
        if 'tempcmds' in config:
            tempcmds_config = config['tempcmds']
        else:
            tempcmds_config = {}
        self.cmd_db = tempcmds.CommandDatabase(config['core']['command_db'],
            config=tempcmds_config, prefix=config['core']['prefix'],
            reply_on_invalid=self._conf_bool('core', 'reply_on_invalid'))

        # Get the "enable_static_cmds" flag
        global static_cmds
        self.static_cmds = self._conf_bool('core', 'enable_static_cmds', True)

        if self.static_cmds:
            if static_cmds is None:
                import lurklite.static_cmds as static_cmds

            # Get the custom commands file
            custom_cmds = config['core'].get('custom_cmds')
            if custom_cmds:
                static_cmds.load_cmd_file(custom_cmds)
        elif 'custom_cmds' in config['core']:
            print('WARNING: A custom commands path is specified, but static co'
                'mmands are disabled! The custom commands will not be loaded.')

        # Get the disable yay/ouch flags
        self.disable_yay  = self._conf_bool('core', 'disable_yay')
        self.disable_ouch = self._conf_bool('core', 'disable_ouch')

        thread_pool = concurrent.futures.ThreadPoolExecutor(32)

        # Get the IRC servers to connect to
        _servers = {}
        kwargs   = None
        for section in config.sections():
            if section == 'irc' or section.startswith('irc.'):
                self._conf_assert(section, 'ip', ('port', int), 'nick',
                    'channels')

                c = config[section]
                kwargs = {}

                for i in 'ident', 'realname', 'ns_identity', 'connect_modes', \
                  'quit_message':
                    if i in c:
                        kwargs[i] = c[i]

                # Add the SSL option
                ssl = None
                if 'tls' in c:
                    ssl = 'tls'
                elif 'ssl' in c:
                    ssl = 'ssl'

                if ssl:
                    kwargs['ssl'] = self._conf_bool(section, ssl)

                # Create the IRC object
                irc = miniirc.IRC(c['ip'], int(c['port']), c['nick'],
                    c['channels'].split(','), auto_connect=False,
                    debug=debug, executor=thread_pool, **kwargs)
                _servers[section] = irc

                # Add the ignores list
                self._add_extras(section, c, irc)

        # Get the Discord bot account (if any)
        if 'discord' in config:
            try:
                import miniirc_discord
            except ImportError:
                err('miniirc_discord is not installed, and a Discord account'
                    ' has been specified in the config file!')

            if getattr(miniirc_discord, 'ver', ()) < (0,5,18):
                print('Support for this version of miniirc_discord will be '
                      'removed in the future.')
                kw = {}
            else:
                kw = {'stateless_mode': True}

            self._conf_assert('discord', 'token')

            c = config['discord']

            # Create the Discord object
            irc = miniirc_discord.Discord(c['token'], 0,
                c.get('nick', '???'), debug=debug, executor=thread_pool, **kw)
            _servers['Discord'] = irc

            # Add the ignores list
            self._add_extras('discord', c, irc)
            self._prefs[irc]['next_update'] = 0

        if 'matrix' in config:
            try:
                import miniirc_matrix
            except ImportError:
                err('miniirc_matrix is not installed, and a Matrix account'
                    ' has been specified in the config file!')

            self._conf_assert('matrix', 'homeserver', 'token')
            c = config['matrix']
            irc = miniirc_matrix.Matrix(c['homeserver'], auto_connect=False,
                                        debug=debug, token=c['token'])
            _servers['Matrix'] = irc
            self._add_extras('matrix', c, irc)

        # Mass connect
        for name, irc in _servers.items():
            irc.debug('Connecting to ' + repr(name) + '...')
            try:
                irc.connect()
            except Exception as exc:
                print(f'Failed to connect to {name!r} - '
                      f'{exc.__class__.__name__}: {exc}')

        irc.debug('Finished connecting to servers!')

    def wait_until_disconnected(self):
        for irc in self._prefs:
            irc.wait_until_disconnected()

# Ensure miniirc isn't outdated
assert miniirc.ver >= (1,8,0), 'lurklite requires miniirc >= v1.8.0!'



#!/usr/bin/python3
#
# Command handler - Processes commands
#

import json, os, re, threading, time, urllib.request, urllib.parse

def web_quote(string):
    return urllib.parse.quote(string, '')

# Try importing msgpack
try:
    import msgpack
except ImportError:
    msgpack = None

# Register "command types"
# You should not use "_hex" for "custom" command types to prevent conflicts
_command_types = {}
_command_ids   = {}
_unknown_regex = []
def register_command_type(type_, use_config=False, *, unknown_re=None,
        _hex=None):
    if unknown_re:
        _unknown_regex.insert(0, (re.compile(unknown_re), type_))

    if type(_hex) == int:
        _command_ids[_hex] = type_

    def n(func):
        if use_config:
            func._tempcmds_config = True
        _command_types[type_] = func
        return func

    return n

# Run a command
def _run_raw_command(cmd_type, code, irc, hostmask, channel, args, *,
        config={}, reply_prefix=None):
    try:
        assert cmd_type in _command_types, 'Invalid command type!'
        handler = _command_types[cmd_type]
        if hasattr(handler, '_tempcmds_config'):
            res = handler(irc, hostmask, channel, code, config, args)
        else:
            res = handler(irc, hostmask, channel, code, args)

        # Sanity check
        assert type(res) == str, 'The command handler did not return a string!'

        # Handle ACTIONs separately
        if res.startswith('\x1b') and res.endswith('\x1b'):
            res    = res[1:-1]
            action = True
        else:
            action = False

        # Make sure the result is a sane length
        if hasattr(irc, 'msglen'):
            maxlen = irc.msglen - 112
        else:
            maxlen = 400

        if len(res) > maxlen:
            res = res[:maxlen] + '...'

        mention = hostmask[0]
        if not mention.endswith('>'):
            mention += ':'

        # Display the output
        if action:
            irc.me(channel, '\u200b' + res)
        elif reply_prefix:
            irc.msg(channel, reply_prefix + mention, res)
        else:
            irc.msg(channel, mention, res)

    except Exception as err:
        irc.notice(channel, '\x034Error running command!\x0f\n' \
            '{}: {}'.format(type(err).__name__, err))
        if irc.debug_file:
            raise

# Command class
class Command:
    type   = 'string'
    config = {}

    def __eq__(self, other):
        return type(self) == type(other) and  self.type == other.type and \
            self.code == other.code

    def as_list(self):
        type_id = self.type

        # Try and use a (more compact) hex code to represent the type
        for _hex in _command_ids:
            if _command_ids[_hex] == type_id:
                type_id = _hex
                break

        return [0, type_id, self.code]

    def as_dict(self):
        return {
            'type': self.type,
            'code': self.code
        }

    def __call__(self, irc, hostmask, args, *, reply_prefix=None):
        return _run_raw_command(self.type, self.code, irc, hostmask, args[0],
            args[1:], config=self.config, reply_prefix=reply_prefix)

    def __init__(self, cmdinfo={}, **kwargs):
        if type(cmdinfo) in (list, tuple) and len(cmdinfo) == 3:
            # cmdinfo: Version (0), type, code
            if cmdinfo[0] == 0:
                cmdinfo = {'type': cmdinfo[1], 'code': cmdinfo[2]}
        elif type(cmdinfo) == str:
            cmdinfo = {'code': cmdinfo}

        cmdinfo.update(kwargs)
        self.code = cmdinfo['code']

        if 'type' in cmdinfo:
            self.type = cmdinfo['type']

            if type(self.type) == int:
                self.type = _command_ids.get(self.type)
        else:
            # Try and guess the command type
            for d in _unknown_regex:
                if d[0].match(self.code):
                    self.type = d[1]
                    break


# Command database
class CommandDatabase:
    _next_update = 0

    def __init__(self, location='commands.db', prefix=None, *,
            reply_on_invalid=False, update_interval=10, config={},
            use_ascii_format=False):
        self.location         = location
        self.reply_on_invalid = reply_on_invalid
        self.prefix           = prefix or '{}|'.format(os.getpid())
        self._config          = config
        self._data            = {}
        self._lock            = threading.Lock()
        self._update_interval = update_interval

        # Note that the database format is auto-detected on load, this only
        # modifies the format used to save the database.
        self.db_format = config.get('db_format', 'msgpack').lower()

    def __repr__(self):
        return 'tempcmds.CommandDatabase(' + repr(self.location) + ')'

    # Update the database
    def _update(self, *, force=False):
        if not force and self._next_update > time.time():
            return

        with self._lock:
            try:
                with open(self.location, 'rb') as f:
                    data = f.read()
                    if not msgpack or data.startswith(b'{'):
                        self._data = json.loads(data.decode('utf-8',
                                                            'replace'))
                    else:
                        self._data = msgpack.loads(data, raw=False)
            except Exception as e:
                print('WARNING: Unable to read commands database!', repr(e))

            self._next_update = time.time() + self._update_interval

    # Get commands
    def get(self, item, *, allowed_aliases=10):
        self._update()
        item = item.lower()
        res  = self._data.get(item)

        # Backwards-compatibility weirdness
        if not res and 'µ' + item in self._data:
            item = 'µ' + item
            res  = self._data[item]

        # Convert the command dict/str to a Command object and resolve aliases.
        if res:
            res = Command(res)
            res.config = self._config

            if res.type == 'alias' and allowed_aliases > 0:
                if res.code.startswith('.'):
                    res.code = res.code[1:]

                return self.get(res.code, allowed_aliases=allowed_aliases - 1)

        return res

    def __getitem__(self, item):
        res = self.get(item)
        if not res:
            raise KeyError(item)
        return res

    def __contains__(self, item):
        self._update()
        item = item.lower()
        return item in self._data or 'µ' + item in self._data

    # Set commands
    def __setitem__(self, item, value):
        if type(value) == dict:
            value = Command(value)

        assert value is None or isinstance(value, Command)

        item = item.lower()
        value = value and value.as_list()

        self._update(force=True)

        with self._lock:
            # Delete "legacy" µcommands
            if 'µ' + item in self._data:
                del self._data['µ' + item]

            if value is None:
                if item in self._data:
                    del self._data[item]
            else:
                self._data[item] = value

            if msgpack and self.db_format != 'json':
                with open(self.location, 'wb') as f:
                    f.write(msgpack.dumps(self._data))
            else:
                with open(self.location, 'w') as f:
                    f.write(json.dumps(self._data))

    # Alias for deleting commands
    def __delitem__(self, item):
        self[item] = None

    # Handle function-like calls
    def __call__(self, irc, hostmask, args, *, reply_prefix=None):
        if args[-1].startswith(self.prefix):
            cmd_args = args[-1].split(' ')
            cmd      = cmd_args[0][len(self.prefix):]
            cmd_args[0] = args[0]
            irc.debug(cmd, cmd_args)

            if cmd in self:
                self[cmd](irc, hostmask, cmd_args, reply_prefix=reply_prefix)
            elif self.reply_on_invalid:
                irc.msg(args[0], f'{hostmask[0]}: Invalid command: {cmd!r}')
            elif irc.debug_file:
                irc.debug(f'User {hostmask} tried to execute invalid command '
                          f'{cmd!r}')

# Handle format strings
@register_command_type('string', _hex=0x00)
def _command_string(irc, hostmask, channel, code, args):
    _a = ' '.join(args)
    try:
        result = code.format(*args, nick=hostmask[0], sender=channel,
            host=hostmask[2], hostmask='{}!{}@{}'.format(*hostmask), args=_a,
            ARGS=_a.upper(), NICK=hostmask[0].upper())
    except IndexError:
        return 'Invalid parameters!'

    return result

# Handle ACTIONs
@register_command_type('action', unknown_re=r'^\*.*\*$', _hex=0x01)
def _command_action(irc, hostmask, channel, code, args):
    if code.startswith('*') and code.endswith('*'):
        code = code[1:-1]

    return _command_string(irc, hostmask, channel, '\x1b' + code + '\x1b', args)

# Display an error if an unknown alias is tried and add aliases to the unknown
#   command RegEx.
@register_command_type('alias', unknown_re=r'^\.', _hex=0x02)
def _command_alias(irc, hostmask, channel, code, args):
    raise RecursionError('Maximum alias recursion depth exceeded.')

# Handle URLs
@register_command_type('url', unknown_re='https://', _hex=0x03)
def _command_url(irc, hostmask, channel, code, args):
    assert code.startswith('http://') or code.startswith('https://')

    code = code.format(*[web_quote(a) for a in args],
        args = web_quote(' '.join(args)), nick = web_quote(hostmask[0]))

    with urllib.request.urlopen(code, timeout=5) as res:
        data = res.read().decode('utf-8', 'replace')

    return data.rstrip('\r\n')

# Remotely execute Python2 lambdas
@register_command_type('lambda', True, unknown_re='lambda', _hex=0x04)
def _command_lambda(irc, hostmask, channel, code, config, args):
    if not code.startswith('lambda'):
        code = 'lambda ' + code

    code = (f'from __future__ import division, generators, nested_scopes,'
            f'print_function, unicode_literals; __builtins__[\'chr\'] = unichr'
            f'; hostmask = {hostmask}; print("|", ({code}){tuple(args)}, "|")')
    lambda_url = config.get('lambda_url',
        'https://tumbolia-two.appspot.com/py/')
    code = lambda_url + web_quote(code)
    res = _command_url(irc, hostmask, channel, code, args)

    # Horrible workaround
    if lambda_url == 'https://tumbolia-two.appspot.com/py/':
        try:
            res = res.encode('latin-1').decode('utf-8')
        except UnicodeError:
            pass

    if res.startswith('TypeError: <lambda>() takes '):
        res = 'Invalid syntax! This command ' + res[22:] + '.'
    elif res.startswith('| ') and res.endswith(' |'):
        res = res[2:-2]

    return res

# Remotely execute node.js functions
@register_command_type('nodejs', True, unknown_re='function', _hex=0x05)
def _command_nodejs(irc, hostmask, channel, code, config, args):
    code = web_quote(f'({code}){tuple(args)}')
    baseurl = config.get('nodejs_url', 'https://untitled-2khw8qubudu1.runkit.sh/')
    code = (f'{baseurl}?code={code}&nick={web_quote(hostmask[0])}'
            f'&channel={web_quote(channel)}&host={web_quote(hostmask[-1])}')
    return _command_url(irc, hostmask, channel, code, args)



#!/usr/bin/python3

from setuptools import setup

with open('README.md', 'r') as f:
    desc = f.read()

setup(
    name        = 'lurklite',
    version     = '0.4.25',
    packages    = ['lurklite'],
    author      = 'luk3yx',
    description = 'A miniirc-based IRC bot.',
    url         = 'https://github.com/luk3yx/lurklite',
    license     = 'AGPLv3',

    entry_points = {'console_scripts': ['lurklite=lurklite.__main__:main']},

    long_description              = desc,
    long_description_content_type = 'text/markdown',
    install_requires              = ['miniirc>=1.8.0', 'msgpack'],
    python_requires               = '>=3.6',

    classifiers = [
        'Development Status :: 5 - Production/Stable',
        'License :: OSI Approved :: GNU Affero General Public License v3',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ]
)
